@@ -1,12 +1,13 @@
 import random
 from string import ascii_letters
-from .arbitrary import Arbitrary
+from .arbitrary import Arbitrary, get_arbitrary, with_samples
 from .utils import roundrobin, filter_possible, unique
 
 
 __all__ = ['const', 'elements', 'oneof', 'bool_', 'char', 'letter']
 
 
+@with_samples
 class elements(Arbitrary):
     """ Choice from a list of constant values. """
 
@@ -20,7 +21,7 @@ class elements(Arbitrary):
     def next_random(self):
         return random.choice(self.el_list)
 
-    def gen_serial(self, amount):
+    def gen_serial(self):
         return self.el_list
 
     def shrink(self, x):
@@ -30,6 +31,7 @@ class elements(Arbitrary):
             yield el
 
 
+@with_samples
 class bool_(elements):
     """ Any :mod:`bool` value (``False`` or ``True``). """
 
@@ -37,13 +39,16 @@ class bool_(elements):
         elements.__init__(self, False, True)
 
 
+@with_samples
 class const(elements):
     """ Any constant value. """
 
+    @Arbitrary.args_for_sample(value='abc')
     def __init__(self, value=None):
         elements.__init__(self, value)
 
 
+@with_samples
 class char(elements):
     """ Any single character (:mod:`string` of length one). """
 
@@ -66,6 +71,7 @@ class char(elements):
         yield chr(255)
 
 
+@with_samples
 class letter(char):
     """ Lower and upper latin letters. """
 
@@ -73,13 +79,15 @@ class letter(char):
         char.__init__(self, ascii_letters)
 
 
+@with_samples
 class oneof(Arbitrary):
     """ Choice (possible weighed) from a list of Arbitraries. """
 
     @Arbitrary.args_for_sample(const('a'), const('b'), const('c'))
     def __init__(self, *arbitraries):
-        self.arbitraries = [arb if isinstance(arb, tuple) else (arb, 1)
-                            for arb in arbitraries]
+        arbitraries = [arb if isinstance(arb, tuple) else (arb, 1)
+                       for arb in arbitraries]
+        self.arbitraries = [(get_arbitrary(arb), w) for arb, w in arbitraries]
 
     def could_generate(self, x):
         return any(arb.could_generate(x) for arb, _ in self.arbitraries)
@@ -88,8 +96,8 @@ class oneof(Arbitrary):
         arb = random.choice_weighted(self.arbitraries)
         return arb.next_random()
 
-    def gen_serial(self, amount):
-        inner_series = [arb.gen_serial(amount) for arb, _ in self.arbitraries]
+    def gen_serial(self):
+        inner_series = [arb.gen_serial() for arb, _ in self.arbitraries]
         return roundrobin(inner_series)
 
     def shrink(self, x):
