@@ -1,10 +1,21 @@
-import itertools
+from itertools import product, islice
 import sys
 import six
-from .arbitraries.arbitrary import Arbitrary
+from .arbitraries.arbitrary import get_arbitrary
+from .arbitraries.sequences import dict_fixkeys
 
 
-__all__ = ['for_all', 'Result', 'ReturnedFalse', 'CheckError']
+__all__ = ['for_all', 'Result', 'ReturnedFalse', 'CheckError',
+           'AssumptionError', 'assume']
+
+
+class AssumptionError(Exception):
+    pass
+
+
+def assume(condition):
+    if not condition:
+        raise AssumptionError
 
 
 class CheckError(Exception):
@@ -110,28 +121,9 @@ class Checker(object):
         self.ntests = ntests
         self.ignore_return = ignore_return
 
-        self.arg_arbs = {}
-        for argname, value in self.args.items():
-            if isinstance(value, type) and not issubclass(value, Arbitrary):
-                # e.g. 'int', 'list' - any class with defined arbitrary_for
-                value = Arbitrary.get_for(value)
-
-            # Arbitrary instanciated above
-            self.arg_arbs[argname] = value
-
-
-    def generate_data(self):
-        each_amount = int(self.ntests ** (1.0 / len(self.arg_arbs)))
-        serial_data = [v.gen_serial(each_amount)
-                       for k, v in self.arg_arbs.items()]
-
-        for values in itertools.product(*serial_data):
-            kwargs = dict(zip(self.arg_arbs.keys(), values))
-            yield kwargs
-
-        for _ in range(self.ntests):
-            kwargs = {k: v.next_random() for k, v in self.arg_arbs.items()}
-            yield kwargs
+        arg_arbs = {argname: get_arbitrary(self.args[argname])
+                    for argname in self.args}
+        self.arb = dict_fixkeys(**arg_arbs)
 
 
     def eval_func(self, data):
@@ -140,28 +132,44 @@ class Checker(object):
             raise ReturnedFalse(func_res)
 
 
-    def full_shrink(self, data):
-        changed = True
-        while changed:
-            changed = False
-            shr_gens = [self.arg_arbs[k].shrink(data[k])
-                        for k in self.arg_arbs.keys()]
-            for shr_values in itertools.product(*shr_gens):
-                shr_data = dict(zip(self.arg_arbs.keys(), shr_values))
+    def full_shrink(self, kwargs):
+        while True:
+            for shr_kwargs in self.arb.shrink(kwargs):
                 try:
-                    self.eval_func(shr_data)
+                    self.eval_func(shr_kwargs)
                 except Exception as e:
-                    data = shr_data
-                    changed = True
-        return data
+                    kwargs = shr_kwargs
+                    break
+            else:
+                break
+        return kwargs
 
 
     def for_all(self):
+        gen_serial = self.arb.gen_serial()
+        def generate(n):
+            if n < self.ntests / 2:
+                try:
+                    return next(gen_serial)
+                except StopIteration:
+                    return self.arb.next_random()
+            else:
+                return self.arb.next_random()
+
         datas = []
-        for kwargs in self.generate_data():
+        n = 0
+        assume_fail_cnt = 0
+        while n < self.ntests:
+            kwargs = generate(n)
             datas.append(kwargs)
             try:
                 self.eval_func(kwargs)
+            except AssumptionError:
+                del datas[-1]
+                assume_fail_cnt += 1
+                if assume_fail_cnt > 10000:
+                    raise RuntimeError('Could not satisfy assume() after 10000 of test cases.')
+                continue
             except Exception as e:
                 kwargs = self.full_shrink(kwargs)
                 try:
@@ -169,11 +177,13 @@ class Checker(object):
                 except Exception as e:
                     _, _, traceback = sys.exc_info()
                     return Result.fail(datas, kwargs, e, traceback)
+            assume_fail_cnt = 0
+            n += 1
 
         return Result.ok(datas)
 
 
-def for_all(func, _ntests=100, **kwargs):
+def for_all(func, _ntests=500, **kwargs):
     """ Test the function ``func`` on ``_ntests`` tests.
 
     For usage examples see :ref:`property-checking`.
